
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <algorithm>
#include "evaluation.h"
#include "../Include/dof.h"

using namespace std;

typedef struct {
    int row;    // row position in grid
    int col;    // col position in grid
    int peak;   // lens position of peak, -1 if unreliable
} DataType;

#include "../Include/High/32x48/peak_backyard_wide.h"
#include "../Include/High/32x48/peak_bars_wide.h"
#include "../Include/High/32x48/peak_books_wide.h"
#include "../Include/High/32x48/peak_building_wide.h"
#include "../Include/High/32x48/peak_cans_wide.h"
#include "../Include/High/32x48/peak_coins_wide.h"
#include "../Include/High/32x48/peak_flowers_wide.h"
#include "../Include/High/32x48/peak_trail_wide.h"

#include "../Include/High/32x48/peak_backyard_narrow.h"
#include "../Include/High/32x48/peak_bars_narrow.h"
#include "../Include/High/32x48/peak_books_narrow.h"
#include "../Include/High/32x48/peak_building_narrow.h"
#include "../Include/High/32x48/peak_cans_narrow.h"
#include "../Include/High/32x48/peak_coins_narrow.h"
#include "../Include/High/32x48/peak_flowers_narrow.h"
#include "../Include/High/32x48/peak_trail_narrow.h"
char const *gridSize = "32x38";

/*
#include "../Include/High/16x24/peak_backyard_wide.h"
#include "../Include/High/16x24/peak_bars_wide.h"
#include "../Include/High/16x24/peak_books_wide.h"
#include "../Include/High/16x24/peak_building_wide.h"
#include "../Include/High/16x24/peak_cans_wide.h"
#include "../Include/High/16x24/peak_coins_wide.h"
#include "../Include/High/16x24/peak_flowers_wide.h"
#include "../Include/High/16x24/peak_trail_wide.h"

#include "../Include/High/16x24/peak_backyard_narrow.h"
#include "../Include/High/16x24/peak_bars_narrow.h"
#include "../Include/High/16x24/peak_books_narrow.h"
#include "../Include/High/16x24/peak_building_narrow.h"
#include "../Include/High/16x24/peak_cans_narrow.h"
#include "../Include/High/16x24/peak_coins_narrow.h"
#include "../Include/High/16x24/peak_flowers_narrow.h"
#include "../Include/High/16x24/peak_trail_narrow.h"
char const *gridSize = "16x24";
*/

DataType *data;

Evaluation::Evaluation( char *benchmark, char *apertureString )
{
    strcpy( this->benchmark, benchmark );
    strcpy( this->apertureString, apertureString );

    bool wide = ( strcmp( apertureString, "wide" ) == 0 );

    /*
     *  ----------------------------------------------------------
     *  backyard &  50 mm & f/1.4, f/8 & landscape, garden setting
     *  bars     &  50 mm & f/1.4, f/8 & architectural, interior
     *  ----------------------------------------------------------
     *  books    &  50 mm & f/1.4, f/8 & macro photography
     *  building &  50 mm & f/2.0, f/8 & landscape, architectural
     *  ----------------------------------------------------------
     *  cans     &  50 mm & f/1.4, f/8 & macro photography
     *  coins    & 200 mm & f/2.8, f/8 & macro photography
     *  ----------------------------------------------------------
     *  flowers  & 200 mm & f/2.8, f/8 & macro photography
     *  trail    &  50 mm & f/4.0, f/8 & landscape, nature
     *  ----------------------------------------------------------
     */

    //double aperture = pow( 2.0, 0.5 );	// 1.4
    //double aperture = pow( 2.0, 1.0 );	// 2.0
    //double aperture = pow( 2.0, 1.5 );	// 2.8
    //double aperture = pow( 2.0, 2.0 );	// 4.0
    //double aperture = pow( 2.0, 3.0 );	// 8.0

    int lens;
    double aperture;
    if( strcmp( benchmark, "backyard" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_backyard_wide;
	    aperture = pow( 2.0, 0.5 );	// 1.4
	}
    	else {
	    data = data_backyard_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "bars" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_bars_wide;
	    aperture = pow( 2.0, 0.5 );	// 1.4
	}
    	else {
	    data = data_bars_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "books" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_books_wide;
	    aperture = pow( 2.0, 0.5 );	// 1.4
	}
    	else {
	    data = data_books_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "building" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_building_wide;
	    aperture = pow( 2.0, 1.0 );	// 2.0
	}
    	else {
	    data = data_building_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "cans" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_cans_wide;
	    aperture = pow( 2.0, 0.5 );	// 1.4
	}
    	else {
	    data = data_cans_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "coins" ) == 0 ) {
	lens = 200;
    	if( wide ) {
	    data = data_coins_wide;
	    aperture = pow( 2.0, 1.5 );	// 2.8
	}
    	else {
	    data = data_coins_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "flowers" ) == 0 ) {
	lens = 200;
    	if( wide ) {
	    data = data_flowers_wide;
	    aperture = pow( 2.0, 1.5 );	// 2.8
	}
    	else {
	    data = data_flowers_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "trail" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_trail_wide;
	    aperture = pow( 2.0, 2.0 );	// 4.0
	}
    	else {
	    data = data_trail_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else {
	printf( "Benchmark not found.\n" );
	exit( 6 );
    }

    double circleOfConfusion = 0.019;

    printf( "%s\n", gridSize );
    printf( "%s\n", benchmark );
    printf( "lens = %3d\n", lens );
    printf( "aperture = %3.1f\n", aperture );

    dof = new DepthOfField( circleOfConfusion, lens, aperture );
}

Evaluation::~Evaluation()
{
}

/*
 *  Percentage of non-zero cells that are covered by
 *  at least one selected image.
 */
void
Evaluation::accuracy( int selected[], int n )
{

    /*
     *  Determine hyperfocal lens position.
     */
    int hyperfocal;
    int hyperNearPosition;
    int hyperFarPosition;
    dof->hyperfocalInLensPositions(
	hyperfocal,
	hyperNearPosition,
	hyperFarPosition
    );
    printf( "hyperfocal = %d: %d <= %d <= %d\n",
	hyperfocal,
	hyperNearPosition,
	hyperfocal,
	hyperFarPosition
    );

    for( int i = 0; i < n; i++ ) {
	int nearPosition;
	int farPosition;
	dof->dofInLensPositions(
	    selected[i],
	    nearPosition,
	    farPosition
	);
	printf( "%-8s:%-6s: selected %3d: dof = [%d, %d]\n",
	    benchmark,
	    apertureString,
	    selected[i],
	    nearPosition,
	    farPosition
	);
    }

    /*
     *  For each non-zero cell, check if that cell is covered
     *  by at least one selected image.
     */
    int nObjects = 0;
    int nCovered = 0;
    int nUnreliable = 0;
    for( int j = 0; data[j].row != -1; j++ ) {
	if( data[j].peak >= 0 ) {

	    /*
	     *  Focal plane is at data[j].peak.
	     *  Determine lens positions off the focal plane
	     *  that are still in acceptable focus.
	     */
	    int nearPosition;
	    int farPosition;
	    dof->dofInLensPositions(
	        data[j].peak, // peak lens position
	        nearPosition,
	        farPosition
	    );

	    /*
	     *  Cell is covered if one of the selected images
	     *  is in acceptable focus.
	     */
	    bool hit = false;
	    for( int i = 0; (i < n) && !hit; i++ ) {
		hit = (nearPosition <= selected[i]) &&
				      (selected[i] <= farPosition);
		/*
		 *  Check for special case where one of the selected
		 *  images is at the hyperfocal lens position and so
		 *  covers the cell.
		 */
		if( !hit && (hyperfocal == selected[i]) ) {
		    hit = (data[j].peak > hyperNearPosition);
		}
	    }

	    if( !hit ) {
		int min = 256;
		int diff;
		for( int i = 0; i < n; i++ ) {
		    diff = nearPosition - selected[i];
		    if( diff < 0 ) diff = -diff;
		    if( min > diff ) min = diff;
		    diff = farPosition - selected[i];
		    if( diff < 0 ) diff = -diff;
		    if( min > diff ) min = diff;
		}
	        printf( "%-8s:%-6s: not covered: %3d, %3d, %3d: dof = [%d, %d]\n",
		    benchmark,
		    apertureString,
		    data[j].row,
		    data[j].col,
		    data[j].peak,
		    nearPosition,
		    farPosition
	        );
	    }

	    if( hit ) nCovered++;
	    nObjects++;
	}
	if( data[j].peak < 0 ) {
	    nUnreliable++;
	}
    }

    printf( "%-8s:%-6s:%3d: unreliable cells: %d\n",
	benchmark,
	apertureString,
	n,
	nUnreliable );
    printf( "%-8s:%-6s:%3d: reliable cells: %d\n",
	benchmark,
	apertureString,
	n,
	nObjects );
    printf( "%-8s:%-6s:%3d: total cells: %d\n",
	benchmark,
	apertureString,
	n,
	nUnreliable + nObjects );
    printf( "%-8s:%-6s:%3d: accuracy = (%4d / %4d) = %8.1f\n\n\n",
	benchmark,
	apertureString,
	n,
	nCovered,
	nObjects,
	(100.0 * nCovered) / (double)nObjects );
}

