/*
 *  Code for evaluating the accuracy of a set of selected images:
 *
 *      Usage: eval benchmark [wide/narrow] input
 *
 *  where input is a file containing the list of lens positions, one per line.
 *
 *  Set grid size in evaluation.cpp: 16x24 or 32x48
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <vector>
#include "evaluation.h"

using namespace std;

int
main(int argc, char *argv[] )
{
    if( argc != 4 ) {
        fprintf( stderr, "Usage: exact benchmark [wide/narrow] input\n" );
        exit( 6 );
    }

    Evaluation eval( argv[1], argv[2] );

    FILE *fp;

    fp = fopen( argv[3], "r" );
    if( fp == NULL ) {
    	fprintf( stderr, "No such file: %s\n", argv[3] );
    	exit( 1 );
    }

    int p;
    int n = 0;
    int m = 0;
    char line[256];
    int selected[256];
    while( fgets( line, 256, fp ) != NULL ) {
	if( strncmp( "Selected images", line, 15 ) == 0 ) {
	    sscanf( line, "Selected images (%d):", &m );
	    break;
	}
    }
    while( fgets( line, 256, fp ) != NULL ) {
	if( line[0] == '\n' ) continue;
	sscanf( line, "%d", &p );
	selected[n] = p;
	n++;
    }
    if( m != n ) {
	printf( "m != n (%d != %d)\n", m, n );
	exit( 1 );
    }

    printf( "%-8s: Selected images (%d):\n", argv[1], n );
    for( int i = 0; i < n; i++ ) {
	printf( "%3d,", selected[i] );
	if( ((i+1) % 10) == 0 ) printf( "\n" );
    }
    printf( "\n" );

    eval.accuracy( selected, n );

    fclose( fp );

    return( 0 );
}
