
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <algorithm>
#include "focusStacking.h"
#include "../Include/dof.h"

using namespace std;

typedef struct {
    int row;    // row position in grid
    int col;    // col position in grid
    int peak;   // lens position of peak, -1 if unreliable
} DataType;

/*
#include "../Include/High/32x48/peak_backyard_wide.h"
#include "../Include/High/32x48/peak_bars_wide.h"
#include "../Include/High/32x48/peak_books_wide.h"
#include "../Include/High/32x48/peak_building_wide.h"
#include "../Include/High/32x48/peak_cans_wide.h"
#include "../Include/High/32x48/peak_coins_wide.h"
#include "../Include/High/32x48/peak_flowers_wide.h"
#include "../Include/High/32x48/peak_trail_wide.h"

#include "../Include/High/32x48/peak_backyard_narrow.h"
#include "../Include/High/32x48/peak_bars_narrow.h"
#include "../Include/High/32x48/peak_books_narrow.h"
#include "../Include/High/32x48/peak_building_narrow.h"
#include "../Include/High/32x48/peak_cans_narrow.h"
#include "../Include/High/32x48/peak_coins_narrow.h"
#include "../Include/High/32x48/peak_flowers_narrow.h"
#include "../Include/High/32x48/peak_trail_narrow.h"
char const *gridSize = "32x38";
*/

#include "../Include/High/16x24/peak_backyard_wide.h"
#include "../Include/High/16x24/peak_bars_wide.h"
#include "../Include/High/16x24/peak_books_wide.h"
#include "../Include/High/16x24/peak_building_wide.h"
#include "../Include/High/16x24/peak_cans_wide.h"
#include "../Include/High/16x24/peak_coins_wide.h"
#include "../Include/High/16x24/peak_flowers_wide.h"
#include "../Include/High/16x24/peak_trail_wide.h"

#include "../Include/High/16x24/peak_backyard_narrow.h"
#include "../Include/High/16x24/peak_bars_narrow.h"
#include "../Include/High/16x24/peak_books_narrow.h"
#include "../Include/High/16x24/peak_building_narrow.h"
#include "../Include/High/16x24/peak_cans_narrow.h"
#include "../Include/High/16x24/peak_coins_narrow.h"
#include "../Include/High/16x24/peak_flowers_narrow.h"
#include "../Include/High/16x24/peak_trail_narrow.h"
char const *gridSize = "16x24";

DataType *data;

FocusStacking::FocusStacking( char *benchmark, char *apertureString )
{
    debug = 0;

    bool wide = ( strcmp( apertureString, "wide" ) == 0 );

    /*
     *  ----------------------------------------------------------
     *  backyard &  50 mm & f/1.4, f/8 & landscape, garden setting
     *  bars     &  50 mm & f/1.4, f/8 & architectural, interior
     *  ----------------------------------------------------------
     *  books    &  50 mm & f/1.4, f/8 & macro photography
     *  building &  50 mm & f/2.0, f/8 & landscape, architectural
     *  ----------------------------------------------------------
     *  cans     &  50 mm & f/1.4, f/8 & macro photography
     *  coins    & 200 mm & f/2.8, f/8 & macro photography
     *  ----------------------------------------------------------
     *  flowers  & 200 mm & f/2.8, f/8 & macro photography
     *  trail    &  50 mm & f/4.0, f/8 & landscape, nature
     *  ----------------------------------------------------------
     */

    //double aperture = pow( 2.0, 0.5 );	// 1.4
    //double aperture = pow( 2.0, 1.0 );	// 2.0
    //double aperture = pow( 2.0, 1.5 );	// 2.8
    //double aperture = pow( 2.0, 2.0 );	// 4.0
    //double aperture = pow( 2.0, 3.0 );	// 8.0

    int lens;
    double aperture;
    if( strcmp( benchmark, "backyard" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_backyard_wide;
	    aperture = pow( 2.0, 0.5 );	// 1.4
	}
    	else {
	    data = data_backyard_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "bars" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_bars_wide;
	    aperture = pow( 2.0, 0.5 );	// 1.4
	}
    	else {
	    data = data_bars_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "books" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_books_wide;
	    aperture = pow( 2.0, 0.5 );	// 1.4
	}
    	else {
	    data = data_books_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "building" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_building_wide;
	    aperture = pow( 2.0, 1.0 );	// 2.0
	}
    	else {
	    data = data_building_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "cans" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_cans_wide;
	    aperture = pow( 2.0, 0.5 );	// 1.4
	}
    	else {
	    data = data_cans_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "coins" ) == 0 ) {
	lens = 200;
    	if( wide ) {
	    data = data_coins_wide;
	    aperture = pow( 2.0, 1.5 );	// 2.8
	}
    	else {
	    data = data_coins_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "flowers" ) == 0 ) {
	lens = 200;
    	if( wide ) {
	    data = data_flowers_wide;
	    aperture = pow( 2.0, 1.5 );	// 2.8
	}
    	else {
	    data = data_flowers_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else if( strcmp( benchmark, "trail" ) == 0 ) {
	lens = 50;
    	if( wide ) {
	    data = data_trail_wide;
	    aperture = pow( 2.0, 2.0 );	// 4.0
	}
    	else {
	    data = data_trail_narrow;
	    aperture = pow( 2.0, 3.0 );	// 8.0
	}
    }
    else {
	printf( "Benchmark not found.\n" );
	exit( 6 );
    }

    double circleOfConfusion = 0.019;

    printf( "%s\n", gridSize );
    printf( "%s\n", benchmark );
    printf( "lens = %3d\n", lens );
    printf( "aperture = %3.1f\n", aperture );

    dof = new DepthOfField( circleOfConfusion, lens, aperture );

    cover = new bool*[4096];		// much larger than needed
    for( int i = 0; i < 4096; i++ ) {
	cover[i] = new bool[maxImages];
	for( int k = 0; k < maxImages; k++ ) {
	    cover[i][k] = false;
	}
    }
}

FocusStacking::~FocusStacking()
{
    for( int i = 0; i < 4096; i++ ) {
	delete[] cover[i];
    }
    delete[] cover;
}

/*
 *  Construct the set covering instance for choosing
 *  the images for focus stacking.
 */
void
FocusStacking::constructSetCoveringInstance()
{
    /*
     *  Fill in the (0,1)-matrix representing the set covering instance.
     *  The rows represent cells, the columns represent images.
     *  There is a 1 in row i, column j if cell i is in acceptably
     *  sharp focus in image j.
     */
    nObjects = 0;
    for( int j = 0; data[j].row != -1; j++ ) {
	if( data[j].peak >= 0 ) {
	    int nearPosition;
	    int farPosition;
	    dof->dofInLensPositions(
		data[j].peak, // peak lens position
		nearPosition,
		farPosition
	    );
	    if( farPosition >= maxImages ) {
		farPosition = maxImages-1;
	    }
	    if( debug ) {
		printf( "%d <= %d <= %d\n",
		    nearPosition, data[j].peak, farPosition );
	    }
	    for( int k = nearPosition; k <= farPosition; k++ ) {
		cover[nObjects][k] = true;
	    }
	    nObjects++;
	}
    }

    /*
     *  Print set covering (0,1)-matrix.
     */
    if( debug ) {
	printf( "\nSet covering (0,1)-matrix:\n\n" );
	for( int i = 0; i < nObjects; i++ ) {
	    for( int k = 0; k < maxImages; k++ ) {
	        printf( "%d", (int)cover[i][k] );
	    }
	    printf( "\n" );
	}
	printf( "\n" );
    }
}

/*
 *  Select images to stack by solving the set covering instance.
 */
vector<int>
FocusStacking::chooseAdaptiveAutofocus( const vector<int> &images )
{
    vector<int> selectedImages;
    bool processed[4096];

    for( int i = 0; i < nObjects; i++ ) {
	processed[i] = false;
    }

    /*
     *  Reduction rule for rows: remove any row that is a
     *  superset of another row.
     */
        int sr[4096];
        int er[4096];
        for( int i = 0; i < nObjects; i++ ) {
            unsigned int k;
            for( k = 0; (cover[i][images[k]] == 0) && (k < images.size()); k++ ) {
                ;
            }
            sr[i] = k;
            for( k = images.size()-1; (cover[i][images[k]] == 0) && (k > 0); k-- ) {
                ;
            }
            er[i] = k;
        }
        for( int i = 0; i < nObjects; i++ )
        for( int j = 0; j < nObjects; j++ ) if( i != j ) {
            if( !processed[j] ) {
                /*
                 *  If row i is a superset, remove it.
                 */
                if( (sr[i] <= sr[j]) && (er[i] >= er[j]) ) {
                    processed[i] = true;
		    if( debug ) {
                        printf("Row %d is removed due to row %d\n", i, j );
		    }
                    break;
                }
            }
        }

    /*
     *  Sweep through images.
     */
    for( unsigned int k = 1; k < images.size(); k++ ) {
        int pl = images[k-1];
        int l = images[k];
        /*
         *  For each object not yet processed ...
         */
        for( int i = 0; i < nObjects; i++ ) if( !processed[i] ) {
            if( !cover[i][l] && cover[i][pl] ) {
		/*
		 *  Add image to set of selected images.
		 */
                selectedImages.push_back( pl );
		if( debug ) {
		    printf( "Selecting image %d\n", pl );
		}
		/*
		 *  Mark any newly covered objects as processed.
		 */
                for( int i2 = 0; i2 < nObjects; i2++ ) {
                    if( cover[i2][pl] ) {
                        processed[i2] = true;
                    }
                }
            }
        }
    }

    /*
     *  Special case at last image.
     */
    int l = images[images.size() - 1];
    for( int i = 0; i < nObjects; i++ ) if( !processed[i] ) {
        if( cover[i][l] ) {
            selectedImages.push_back( l );
	    if( debug ) {
	        printf( "Selecting image %d\n", l );
	    }
            for( int i2 = 0; i2 < nObjects; i2++ ) {
                if( cover[i2][l] ) {
                    processed[i2] = true;
                }
            }
        }
    }

    return( selectedImages );
}

/*
 *  Remove images that do not need to be looked at.
 */
vector<int>
FocusStacking::shrinkImageSet()
{
    vector<int> imagesToExamine;

    /*
     *  Reduction rule for columns: remove any column that is a
     *  subset of another column.
     */
    bool include[maxImages];
    for( int i = 0; i < maxImages; i++ ) {
	include[i] = true;
    }

    for( int i = 0; i < maxImages; i++ )
    for( int j = 0; j < maxImages; j++ ) if( i != j ) {
        if( include[j] ) {
            /*
             *  If column i is a subset of column j, remove column i.
             */
            bool subset = true;
            for( int k = 0; k < nObjects && subset; k++ ) {
                if( cover[k][i] > cover[k][j] ) {
                    subset = false;
                }
            }
            if( subset ) {
                include[i] = false;
                if( debug ) {
                    printf("Column %d is removed due to column %d\n", i, j );
                }
                break;
            }
        }
    }

    for( int k = 0; k < maxImages; k++ ) {
	if( include[k] ) {
	    if( debug ) {
	        printf( "image to consider: %d\n", k );
	    }
	    imagesToExamine.push_back( k );
	}
    }

    if( debug ) {
        printf( "total images to consider: %d\n", imagesToExamine.size() );
    }

    return( imagesToExamine );
}

/*
 *  Select images to stack using new adaptive autofocus algorithm.
 */
vector<int>
FocusStacking::selectAdaptiveAutofocus()
{
    constructSetCoveringInstance();
    vector<int> selectedImages;
    selectedImages = chooseAdaptiveAutofocus( shrinkImageSet() );

    return( selectedImages );
}

