/*
 *  This class takes in a grid of focus measures and can be used
 *  to choose images based on various selection algorithms.
 */
#ifndef _FOCUSSTACKING_H
#define _FOCUSSTACKING_H

#include <vector>

class DepthOfField;

class FocusStacking
{
    public:

	FocusStacking( char *benchmark, char *apertureString );
	~FocusStacking();

	/*
	 *  Select images to stack using new Adaptive Autofocus
	 *  algorithm.
	 */
	std::vector<int> selectAdaptiveAutofocus();

    private:
	static const int maxImages = 256;

	int nObjects;

	int debug;

	/*
	 *  cover[i][k] == 1 if the i^th cell is sharp in image k,
	 *		   0 otherwise.
	 */
	bool **cover;

	/*
	 *  Depth of field calculations and mapping to lens positions.
	 */
	DepthOfField *dof;

	void constructSetCoveringInstance();

	/*
	 *  Remove images that do not need to be looked at.
	 */
	std::vector<int> shrinkImageSet();

	/*
	 *  Given a set of images, performs the adaptive autofocus 
	 *  algorithm (used by selectAdaptiveAutofocus).
	 */
	std::vector<int> chooseAdaptiveAutofocus( const std::vector<int>& );
};

#endif // _FOCUSSTACKING_H
