
typedef struct {
    int row;    // row position in grid
    int col;    // col position in grid
    int peak;   // lens position of peak, -1 if unreliable
} DataType;

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "peak_backyard_wide.h"
#include "peak_bars_wide.h"
#include "peak_books_wide.h"
#include "peak_building_wide.h"
#include "peak_cans_wide.h"
#include "peak_coins_wide.h"
#include "peak_flowers_wide.h"
#include "peak_trail_wide.h"

#include "peak_backyard_narrow.h"
#include "peak_bars_narrow.h"
#include "peak_books_narrow.h"
#include "peak_building_narrow.h"
#include "peak_cans_narrow.h"
#include "peak_coins_narrow.h"
#include "peak_flowers_narrow.h"
#include "peak_trail_narrow.h"

void generate( DataType data[] );

main(int argc, char *argv[] )
{
    if( argc != 3 ) {
        fprintf( stderr, "Usage: count benchmark [wide/narrow]\n" );
        exit( 6 );
    }

    DataType *data;

    bool wide = ( strcmp( argv[2], "wide" ) == 0 );

    if( strcmp( argv[1], "backyard" ) == 0 ) {
    	if( wide ) { data = data_backyard_wide; }
    	else	   { data = data_backyard_narrow; }
    }
    else if( strcmp( argv[1], "bars" ) == 0 ) {
    	if( wide ) { data = data_bars_wide; }
    	else	   { data = data_bars_narrow; }
    }
    else if( strcmp( argv[1], "books" ) == 0 ) {
    	if( wide ) { data = data_books_wide; }
    	else	   { data = data_books_narrow; }
    }
    else if( strcmp( argv[1], "building" ) == 0 ) {
    	if( wide ) { data = data_building_wide; }
    	else	   { data = data_building_narrow; }
    }
    else if( strcmp( argv[1], "cans" ) == 0 ) {
    	if( wide ) { data = data_cans_wide; }
    	else	   { data = data_cans_narrow; }
    }
    else if( strcmp( argv[1], "coins" ) == 0 ) {
    	if( wide ) { data = data_coins_wide; }
    	else	   { data = data_coins_narrow; }
    }
    else if( strcmp( argv[1], "flowers" ) == 0 ) {
    	if( wide ) { data = data_flowers_wide; }
    	else	   { data = data_flowers_narrow; }
    }
    else if( strcmp( argv[1], "trail" ) == 0 ) {
    	if( wide ) { data = data_trail_wide; }
    	else	   { data = data_trail_narrow; }
    }
    else {
	printf( "Benchmark not found.\n" );
	exit( 6 );
    }

    generate( data );
}


void
generate( DataType data[] )
{
    int k;
    int unreliable = 0;
    int depths[256];

    for( k = 0; k < 256; k++ ) {
	depths[k] = 0;
    }

    k = 0;
    for( int i = 0; i < 32; i++ )
    for( int j = 0; j < 48; j++ ) {
	if( (data[k].row != i) ||
	    (data[k].col != j) ) {
	    printf( "trouble\n" );
	    exit( 6 );
	}
	if( data[k].peak >= 0 ) {
	    depths[data[k].peak]++;
	}
	else {
	    unreliable++;
	}
	k++;
    }

    printf( "unreliable = %d\n", unreliable );
    for( k = 0; k < 256; k++ ) {
	if( depths[k] != 0 ) {
	    printf( "depth[%3d] = %3d\n", k, depths[k] );
	}
    }
}

