/*
 *  Routines for depth of field calculations and mapping to lens positions.
 */
#ifndef _DOF_H_
#define _DOF_H_

class DepthOfField
{
    public:
	DepthOfField( double circleOfConfusion, int focalLength, double aperture );
	~DepthOfField();

	/*
	 *  Circle of confusion values.
	 */
	static const double Full  = 0.030;
	static const double APS_H = 0.023;
	static const double APS_C = 0.019;

	void dofInLensPositions(
	    int    lensPosition,
	    int    &nearPosition,
	    int    &farPosition
	);

	void hyperfocalInLensPositions(
	    int    &lensPosition,
	    int    &nearPosition,
	    int    &farPosition
	);

	int maxLensPosition();

    protected:
	typedef struct {
	    int position;	// lens position
	    double distance;	// distance mark on lens expressed in meters
	} DataType;

	static const int MAX_LensPositions_50mm;
	static const DataType data_50mm[];

	static const int MAX_LensPositions_200mm;
	static const DataType data_200mm[];

	static const double infinity;

	double lens[256];
	int nPos;		// max lens positions
	double circleOfConfusion; 
	double aperture;
	int focalLength;

	void dofInMeters(
	    double subjectDistance,
	    double &nearDistance,
	    double &farDistance
	);

	void interpolate( const DataType data[], double lens[] );
};

#endif // _DOF_H_
