/*
 *  For each global max, print the feature values and the correct
 *  label for that peak.
 */
#ifndef _LABEL_H
#define _LABEL_H

typedef struct {
    int row;	// row position in 4x6 grid
    int col;	// col position in 4x6 grid
    int range;	// boolean indicating range of values (1), or a disjuction (0)
    int l;	// lower value (of range or disjunction)
    int u;	// upper value (of range or disjunction)
} DataType;

class Label
{
    public:
	Label( int nRows, int nCols, int nImages, int option = 0 );
	~Label();

	void generateData(
		char *benchmark,
		double ***measures );

    private:
	int nRows;
	int nCols;
	int nImages;
	int option;
	int rowFactor;
	int colFactor;

	bool lookup(
		char *benchmark,
		int row,
		int col,
		int peak,
		DataType *data );

	bool determineLabel(
		char *benchmark,
		int peak,
		int row,
		int col );

	void printFeatures(
		char *benchmark,
		int row,
		int col,
		int **depthMap,
		double ***measures );

	double getMeasure(
		int i,
		int j,
		int k,
		double ***measures );

	double getCellMeasure(
		int i,
		int j,
		int k,
		double ***measures );

	/*
	 *  depthMap[i][j] is the estimated depth (in lens positions)
	 *  of cell (i,j).
	 */
	int **depthMap;
};

#endif // _LABEL_H
