
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <vector>
#include "../Include/focusMeasure.h"
#include "label.h"

using namespace std;

void readGray( char *fileName, int n, uchar *buffer );

int
main(int argc, char *argv[] )
{
    if( argc <= 2 ) {
        fprintf( stderr, "Usage: label benchmark fileName0.gray ...\n" );
        exit( 6 );
    }

    /*
     *  Pixel dimensions of low resolutions images.
     */
    const int w = 1056;
    const int h =  704;
    const int n = w * h;
    uchar buffer[n];

    int nRows;
    int nCols;
    //nRows =  8; nCols = 12;
    //nRows = 16; nCols = 24;
    nRows = 32; nCols = 48;
    int apply = 2;	// 0 = squared gradient, 2 = nokia
    int option = 0;	// 0 = no smoothing, 1 = smoothing

    double ***measures = new double**[nRows];
    for( int i = 0; i < nRows; i++ ) {
	measures[i] = new double*[nCols];
	for( int j = 0; j < nCols; j++ ) {
	    measures[i][j] = new double[argc];
	}
    }

    FocusMeasure focus;

    /*
     *  Read in images (gray values) and compute focus measures.
     */
    double v = 0;
    double min = HUGE_VAL;
    double max = 0;
    int gridWidth = w / nCols;
    int gridHeight = h / nRows;
    for( int k = 2; k < argc; k++ ) {
        readGray( argv[k], n, buffer );
        for( int i = 0; i < nRows; i++ )
        for( int j = 0; j < nCols; j++ ) {
	    switch( apply ) {
		case  0: v = focus.squaredGradient(
				buffer,
				w,
				h,
				j * gridWidth,
				i * gridHeight,
				gridWidth,
				gridHeight
			     );
			 break;
		case  1: v = focus.brenner(
				buffer,
				w,
				h,
				j * gridWidth,
				i * gridHeight,
				gridWidth,
				gridHeight
			 );
			 break;
		case  2: v = focus.nokiaMeasure(
				buffer,
				w,
				h,
				j * gridWidth,
				i * gridHeight,
				gridWidth,
				gridHeight
			  );
			  break;
	    }
	    if( max < v ) {
		max = v;
	    }
	    if( min > v ) {
		min = v;
	    }
	    measures[i][j][k-2] = (double)v;
        }
    }

    /*
     *  Normalize focus measures.
     */
    for( int i = 0; i < nRows; i++ )
    for( int j = 0; j < nCols; j++ ) {
	for( int k = 0; k < argc-2; k++ ) {
	    measures[i][j][k] = (measures[i][j][k] - min)/(double)(max - min);
        }
    }

    /*
     *  Generate the feature values and label the instance.
     */
    Label label( nRows, nCols, argc-2, option );
    label.generateData( argv[1], measures );

    /*
     *  Cleanup.
     */
    for( int i = 0; i < nRows; i++ ) {
        for( int j = 0; j < nCols; j++ ) {
            delete[] measures[i][j];
        }
        delete[] measures[i];
    }
    delete[] measures;

    return( 0 );
}

/*
 *  Read the gray values from a file into buffer.
 */
void
readGray( char *fileName, int n, uchar *buffer )
{
    FILE *fp;

    fp = fopen( fileName, "rb" );
    if( fp == NULL ) {
    	fprintf( stderr, "No such file: %s\n", fileName );
    	exit( 1 );
    }

    int result = fread( buffer, 1, n, fp );
    if( result != n ) {
    	fprintf( stderr, "Wrong number of bytes: %s\n", fileName );
    	exit( 1 );
    }

    fclose( fp );
}

